#!/usr/local/bin/python
# -*- coding: iso-8859-1 -*-

## Copyright 2005 by LivingLogic AG, Bayreuth/Germany.
## Copyright 2005 by Walter Drwald
##
## All Rights Reserved
##
## See orasql.py for the license


import sys

from ll import orasql


if __name__ == "__main__":
	import sys, optparse

	p = optparse.OptionParser(usage="usage: %prog [options] connectstring >output.sql")
	fks = ("keep", "disable", "drop")
	p.add_option("-v", "--verbose", dest="verbose", help="Give a progress report?", default=False, action="store_true")
	p.add_option("-f", "--fks", dest="fks", help="How should foreign keys from other schemas be treated (%s)?" % ", ".join(fks), default="disable", choices=fks)
	p.add_option("-x", "--execute", dest="execute", action="store_true", help="immediately execute the commands instead of printing them?")
	p.add_option("-k", "--keepdollar", dest="keepdollar", help="Output objects with '$' in their name?", default=False, action="store_true")

	(options, args) = p.parse_args()
	if len(args) != 1:
		p.error("incorrect number of arguments")
		sys.exit(1)

	connection = orasql.connect(args[0])
	cursor = connection.cursor()

	term = not options.execute
	
	ddls = []
	for definition in cursor.iterobjects(mode="drop", all=False):
		# Get DDL
		ddl = ""
		action = "skipped"
		if definition.owner is not None:
			if isinstance(definition, orasql.FKDefinition):
				if options.fks == "disable":
					ddl = definition.disableddl(cursor, term)
					action = "disabled"
				elif options.fks == "drop":
					ddl = definition.dropddl(cursor, term)
					action = None
		elif "$" not in definition.name or options.keepdollar:
			ddl = definition.dropddl(cursor, term)
			action = None

		# Progress report
		if options.verbose:
			msg = "%s: %s: %r" % (sys.argv[0], cursor.connection.connectstring(), definition)
			if action is not None:
				msg += " (%s)" % action
			print >>sys.stderr, msg

		if ddl:
			# Print or execute DDL
			if options.execute:
				ddls.append(ddl)
			else:
				sys.stdout.write(ddl)

	# Execute DDL
	if options.execute:
		for ddl in ddls:
			cursor.execute(ddl)
